"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const micro_1 = require("micro");
const aws_serverless_micro_1 = __importDefault(require("aws-serverless-micro"));
const pino_http_1 = __importDefault(require("pino-http"));
const micro_boom_1 = require("micro-boom");
const bytes_1 = __importDefault(require("bytes"));
const { LIGHT_ENVIRONMENT, NODE_ENV } = process.env;
const isProd = NODE_ENV === 'production';
const isNetlify = LIGHT_ENVIRONMENT === 'netlify';
const isAWS = LIGHT_ENVIRONMENT === 'aws';
const isRunKit = LIGHT_ENVIRONMENT === 'runkit';
let Youch;
let forTerminal;
/* istanbul ignore next */
if (!isProd) {
    Youch = require('youch'); // eslint-disable-line
    forTerminal = require('youch-terminal'); // eslint-disable-line
}
let logger;
/* istanbul ignore next */
if (isProd) {
    const pinoHandler = pino_http_1.default();
    logger = (fn) => async (req, res) => {
        pinoHandler(req, res);
        return fn(req, res);
    };
}
else {
    const signale = require('./utils/logger'); // eslint-disable-line
    logger = (fn) => async (req, res) => {
        const id = Math.random().toString(36).substring(2, 6);
        const log = signale.scope(id);
        const msg = `${req.url}`;
        const { method } = req;
        if (log[method]) {
            log[method](msg);
        }
        else {
            log.request(msg);
        }
        const done = () => {
            res.removeListener('finish', onfinish); // eslint-disable-line
            res.removeListener('close', onclose); // eslint-disable-line
            const status = res.statusCode;
            const len = res._contentLength; // eslint-disable-line
            let length;
            if ([204, 205, 304].includes(status)) {
                length = '';
            }
            else if (len == null) {
                length = '-';
            }
            else {
                length = bytes_1.default(len).toLowerCase();
            }
            log.response(`${status} ${length}`);
        };
        const onfinish = done.bind(null, 'finish');
        const onclose = done.bind(null, 'close');
        res.once('finish', onfinish);
        res.once('close', onclose);
        return fn(req, res);
    };
}
const youchErrors = (fun) => async (req, res) => {
    try {
        return await fun(req, res);
    }
    catch (err) {
        const youch = new Youch(err, req);
        const json = await youch.toJSON();
        console.log(forTerminal(json)); // eslint-disable-line
        throw err;
    }
};
exports.default = (route) => {
    const fn = async (Req, Res) => {
        let exec = async (req, res) => {
            const middleware = route.middleware || [];
            for (const mw of middleware) { // eslint-disable-line
                await mw(req, res); // eslint-disable-line
                if (res.headersSent) {
                    return null;
                }
            }
            return route.handler(req, res);
        };
        const plugins = route.plugins || [];
        if (fn.log !== false) {
            plugins.unshift(logger);
            fn.log = false;
        }
        /* istanbul ignore next */
        if (!isProd) {
            plugins.unshift(youchErrors);
        }
        plugins.unshift(micro_boom_1.handleErrors);
        if (plugins.length) {
            exec = plugins.reverse().reduce((acc, val) => val(acc), exec);
        }
        return exec(Req, Res);
    };
    Object.keys(route).forEach((key) => {
        fn[key] = route[key];
    });
    fn.log = true;
    fn.module = __dirname;
    fn.handler = fn;
    /* istanbul ignore next */
    if (!isNetlify && !isAWS) {
        fn.handler = async (req, res) => micro_1.run(req, res, fn);
    }
    /* istanbul ignore if */
    if (isNetlify || isAWS) {
        return {
            handler: aws_serverless_micro_1.default(fn),
        };
    }
    if (isRunKit) {
        return {
            endpoint: fn,
        };
    }
    return fn;
};
